var lodash = require('lodash');
var SHA1 = require('../lib/sha1');

var subSHA1Password = function (password) {
    var sha1_password = password.length > 8 ? SHA1(password) : password;
    return subSHA1(sha1_password);
};

var subSHA1 = function (sha1_password) {
    var len = sha1_password.length;
    return sha1_password.substring(len - 8, len);
};

var xor = function (one, other) {
    var r = "";
    for (var i = 0; i < one.length && i < other.length; ++i) {
        r += String.fromCharCode(one.charCodeAt(i) ^ other.charCodeAt(i));
    }
    return r;
};

var toHex = function (one) {
    var hex = '', tmp;
    for (var i = 0; i < one.length; i++) {
        tmp = one.charCodeAt(i).toString(16)
        if (tmp.length == 1) {
            tmp = '0' + tmp;
        }
        hex += tmp
    }
    return hex;
};

var generateCode = function (str, sha1_password) {
    return toHex(
        xor(
            SHA1(str || ''),
            sha1_password || ''
        )
    ).substring(0, 8);
};

var generateExpCode = function (sha1_password, exps) {
    var switches_str = generateSwitchesStr(exps);
    return generateCode(switches_str, sha1_password);
};

var generateSwitchesStr = function (exps) {
    return lodash.sortBy(
        lodash.map(
            exps,
            function (exp) {
                return exp.name + '=' + exp.value;
            })
    ).join(',');
};

var serverExpCode = function (exps, password) {
    exps = lodash.map(exps, function (v, k) {
        var obj = {
            name: k,
            value: v
        };
        return obj;
    });
    return encodeURIComponent(
        generateSwitchesStr(exps) + ',' + generateExpCode(subSHA1Password(password), exps)
    );
};

module.exports = serverExpCode;